<?php

namespace SupabloxPro;

const ASSETS_DIR = DIR . '/build';
const ASSETS_URL = URL . 'build/';

function register_assets() {
	if ( is_development() ) {
		register_script( 'supablox-pro/runtime', 'runtime.js' );
	}

	register_script( 'supablox-pro/block-editor', 'block-editor.js' );
	register_style( 'supablox-pro/block-editor', 'block-editor.css' );

	register_script( 'supablox-pro/block-supports', 'block-supports.js' );
	register_style( 'supablox-pro/block-supports', 'block-supports.css' );

	register_script( 'supablox-pro/blocks', 'blocks.js' );
	register_style( 'supablox-pro/blocks', 'blocks.css' );

	register_script( 'supablox-pro/public', 'public.js' );
	register_style( 'supablox-pro/public', 'public.css' );
	wp_style_add_data( 'supablox-pro/public', 'path', ASSETS_DIR . '/public.css' );

	register_script( 'supablox-pro/integrations/supablox', 'integrations/supablox.js' );
	register_style( 'supablox-pro/integrations/supablox', 'integrations/supablox.css' );
}
add_action( 'init', __NAMESPACE__ . '\register_assets', 0 );

function register_script( string $handle, string $filename, array $dependencies = [] ) {
	$assets_info = get_asset_info( $filename );

	if ( $handle !== 'supablox-pro/runtime' && is_development() ) {
		$dependencies[] = 'supablox-pro/runtime';
	}

	wp_register_script(
		$handle,
		ASSETS_URL . $filename,
		array_merge( $assets_info['dependencies'], $dependencies ),
		$assets_info['version'],
		true
	);
}

function register_style( string $handle, string $filename, array $dependencies = [] ) {
	$version = VERSION;

	if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) {
		if ( file_exists( ASSETS_DIR . "/$filename" ) ) {
			$version = filemtime( ASSETS_DIR . "/$filename" );
		}
	}

	wp_register_style(
		$handle,
		ASSETS_URL . $filename,
		$dependencies,
		$version
	);
}

function get_asset_info( string $filename ) {
	$asset = str_replace( '.js', '', $filename );
	$path = ASSETS_DIR . "/$asset.asset.php";

	if ( file_exists( $path ) ) {
		return require $path;
	}

	return [
		'dependencies' => [],
		'version'      => VERSION,
	];
}
