<?php

namespace SupabloxPro;

const BLOCK_STYLE_POST_TYPE = 'supablox_block_style';
const BLOCK_STYLE_META_KEY = 'block_names';

function register_block_style_post_type() {
	register_post_type(
		BLOCK_STYLE_POST_TYPE,
		[
			'public' => false,
			'show_in_rest' => true,
			'show_in_menu' => false,
			'rest_base' => 'block-styles',
			'rest_namespace' => 'supablox-pro/v1',
			'late_route_registration' => true,
			'map_meta_cap' => true,
			'capabilities' => [
				'read' => 'edit_posts',
				'create_posts' => 'edit_theme_options',
				'edit_posts' => 'edit_theme_options',
				'edit_published_posts' => 'edit_theme_options',
				'delete_published_posts' => 'edit_theme_options',
				'edit_others_posts' => 'edit_theme_options',
				'delete_others_posts' => 'edit_theme_options',
			],
			'supports' => [
				'title',
				'custom-fields'
			],
		]
	);

	register_post_meta(
		BLOCK_STYLE_POST_TYPE,
		BLOCK_STYLE_META_KEY,
		[
			'type' => 'string',
			'single' => true,
		]
	);
}
add_action( 'init', __NAMESPACE__ . '\register_block_style_post_type' );

function init_block_styles() {
	register_block_styles( get_block_styles() );
}
add_action( 'init', __NAMESPACE__ . '\init_block_styles' );

function register_block_styles_rest_route() {
	register_rest_field(
		BLOCK_STYLE_POST_TYPE,
		BLOCK_STYLE_META_KEY,
		[
			'get_callback' => function ( array $params ) {
				if ( isset( $params['id'] ) ) {
					if ( $value = get_post_meta( $params['id'], BLOCK_STYLE_META_KEY, true ) ) {
						if ( is_array( $value ) ) {
							return $value;
						}
						return json_decode( $value );
					}
				}
				return [];
			},
			'update_callback' => function ( array $block_types, \WP_Post $post ) {
				update_post_meta( $post->ID, BLOCK_STYLE_META_KEY, wp_json_encode( $block_types ) );
			},
			'schema' => [
				'type' => 'array',
				'items' => [
					'type' => 'string',
				],
			],
		]
	);
}
add_action( 'rest_api_init', __NAMESPACE__ . '\register_block_styles_rest_route' );

function register_block_styles( array $block_styles ) {
	foreach ( $block_styles as $name => $style ) {
		register_block_style(
			$style['block_names'],
			[
				'name' => $name,
				'label' => $style['label'],
				'style_data' => is_admin() ? [ 'css' => '' ] : [],
			]
		);
	}
}

function get_block_styles() {
	global $wpdb;

	$_results = wp_cache_get( 'block_styles', 'supablox' );

	if ( ! $_results ) {
		$_results = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT p.post_name, p.post_title, p.post_content, pm.meta_value as block_names
				FROM {$wpdb->posts} p
				LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id
				WHERE p.post_type = %s
				AND p.post_status = 'publish'
				AND pm.meta_key = %s
				ORDER BY p.post_name ASC",
				BLOCK_STYLE_POST_TYPE,
				BLOCK_STYLE_META_KEY
			)
		) ?: [];

		wp_cache_add( 'block_styles', $_results, 'supablox' );
	}

	$block_styles = [];

	foreach ( $_results as $result ) {
		$block_styles[ $result->post_name ] = [
			'label' => $result->post_title,
			'inline_style' => $result->post_content,
			'block_names' => $result->block_names ? json_decode( $result->block_names ) : [],
		];
	}

	return $block_styles;
}
