<?php

namespace SupabloxPro\Block_Supports;

\WP_Block_Supports::get_instance()->register(
	'supabloxProEffects',
	[
		'register_attribute' => __NAMESPACE__ . '\register_effects_support',
	]
);

function register_effects_support( \WP_Block_Type $block_type ) {
	if ( ! block_has_support( $block_type, 'supabloxProEffects', false ) ) {
		return;
	}

	$support = $block_type->supports[ 'supabloxProEffects' ];

	if ( $support === true ) {
		$support = [
			'transition' => true,
		];
	}

	if ( $support['transition'] !== false ) {
		register_transition_effect_attributes( $block_type );
	}

	style_engine()->register(
		$block_type,
		[
			'color' => [
				'type' => 'string',
				'default' => '',
				'property' => 'color',
				'responsive' => false,
				'output' => false,
			],
			'backgroundColor' => [
				'type' => 'string',
				'default' => '',
				'property' => 'background-color',
				'responsive' => false,
				'output' => false,
			],
		]
	);
}

function register_transition_effect_attributes( \WP_Block_Type $block_type ) {
	$class_name = 'has-transition';
	$attributes = [
		'transitionTimingFunction' => [
			'type' => 'string',
			'default' => 'linear',
			'output' => false,
			'responsive' => false,
			'property' => '--sb-transition-timing-function',
			'class_name' => $class_name,
		],
		'transitionDuration' => [
			'type' => 'string',
			'default' => '0ms',
			'output' => false,
			'responsive' => false,
			'property' => '--sb-transition-duration',
			'class_name' => $class_name,
		],
		'transitionDelay' => [
			'type' => 'string',
			'default' => '0ms',
			'output' => false,
			'responsive' => false,
			'property' => '--sb-transition-delay',
			'class_name' => $class_name,
		],
	];

	style_engine()->register( $block_type, $attributes );
}
