<?php

namespace SupabloxPro\Block_Supports;

\WP_Block_Supports::get_instance()->register(
	'supabloxProStyle',
	[
		'register_attribute' => __NAMESPACE__ . '\register_style_support',
		'apply' => __NAMESPACE__ . '\apply_style_support',
	]
);

function register_style_support( \WP_Block_Type $block_type ) {
	$block_support_name = 'supabloxProStyle';

	if ( ! block_has_support( $block_type, $block_support_name, false ) ) {
		return;
	}

	$attributes = [];

	foreach ( $block_type->supports[ $block_support_name ] as $attr_name => $attr_config ) {
		if ( $attr_config === true ) {
			if ( isset( $block_type->attributes[ $attr_name ] ) ) {
				$type = $block_type->attributes[ $attr_name ]['type'] ?? 'string';
				$default = $block_type->attributes[ $attr_name ]['default'] ?? null;

				$attributes[ $attr_name ] = [
					'type' => $type,
					'default' => $default,
					'output' => false,
				];
			}
		} elseif ( is_array( $attr_config ) ) {
			$attributes[ $attr_name ] = wp_parse_args( $attr_config, [ 'output' => false ] );
		}
	}

	style_engine()->register( $block_type, $attributes );
}

function apply_style_support( \WP_Block_Type $block_type ) {
	global $_supablox_pro_block_support_style_class_name;

	if (
		isset( $block_type->attributes['supablox'] ) &&
		! isset( $_supablox_pro_block_support_style_class_name['supablox'] )
	) {
		$_supablox_pro_block_support_style_class_name['supablox'] = true;

		return [
			'class' => 'sb-block',
		];
	}

	return [];
}
