<?php

namespace SupabloxPro\Block_Supports;

\WP_Block_Supports::get_instance()->register(
	'supabloxProTypography',
	[
		'register_attribute' => __NAMESPACE__ . '\register_typography_support',
		'apply' => __NAMESPACE__ . '\apply_typography_support',
	]
);

add_filter( 'render_block_data', __NAMESPACE__ . '\typography_render_block_data' );

function register_typography_support( \WP_Block_Type $block_type ) {
	if ( has_typography_support( $block_type ) ) {
		if ( ! array_key_exists( 'fontSize', $block_type->attributes ) ) {
			$block_type->attributes['fontSize'] = [ 'type' => 'string' ];
		}

		if ( ! array_key_exists( 'style', $block_type->attributes ) ) {
			$block_type->attributes['style'] = [ 'type' => 'object' ];
		}

		style_engine()->register(
			$block_type,
			[
				'fontSize' => [
					'type' => 'string',
					'responsive' => true,
					'output' => false,
					'property' => 'font-size',
					'should_output_callback' => __NAMESPACE__ . '\should_output_typography_font_size',
				],
			]
		);
	}
}

function apply_typography_support( \WP_Block_Type $block_type, $block_attributes ) {
	if ( ! has_typography_support( $block_type ) ) {
		return [];
	}

	$attributes = [];
	$typography_block_styles = [];

	if ( ! should_skip_block_supports_serialization( $block_type, 'supabloxProTypography', 'fontSize' ) ) {
		$preset_font_size = array_key_exists( 'fontSize', $block_attributes )
			? "var:preset|font-size|{$block_attributes['fontSize']}"
			: null;
		$custom_font_size = isset( $block_attributes['style']['typography']['fontSize'] )
			? $block_attributes['style']['typography']['fontSize']
			: null;
		$typography_block_styles['fontSize'] = $preset_font_size
			? $preset_font_size
			: wp_get_typography_font_size_value( [ 'size' => $custom_font_size ] );
	}

	if ( empty( $typography_block_styles ) ) {
		return $attributes;
	}

	$styles = wp_style_engine_get_styles(
		[ 'typography' => $typography_block_styles ],
		[ 'convert_vars_to_classnames' => true ]
	);

	if ( ! empty( $styles['classnames'] ) ) {
		$attributes['class'] = $styles['classnames'];
	}

	if ( ! empty( $styles['css'] ) ) {
		$attributes['style'] = $styles['css'];
	}

	return $attributes;
}

/**
 * Don't output the `font-size` property for default breakpoint and state.
 * WordPress already add a `has-*-font-size` class or inline style.
 */
function should_output_typography_font_size( $value, string $breakpoint, string $state ) {
	return ! ( $breakpoint === 'lg' && $state === 'default' );
}

function typography_render_block_data( array $parsed_block ) {
	if (
		isset( $parsed_block['blockName'] ) &&
		has_typography_support( $parsed_block['blockName'] ) &&
		empty( $parsed_block['attrs']['fontSize'] ) &&
		! empty( $parsed_block['attrs']['style']['typography']['fontSize'] )
	) {
		$parsed_block['attrs']['fontSize'] = $parsed_block['attrs']['style']['typography']['fontSize'];
	}
	return $parsed_block;
}

function has_typography_support( \WP_Block_Type|string|null $block_type ) {
	if ( is_string( $block_type ) ) {
		$block_type = \WP_Block_Type_Registry::get_instance()->get_registered( $block_type );
	}
	return (
		block_has_support( $block_type, 'supabloxProTypography', false ) &&
		block_has_support( $block_type, 'typography' )
	);
}
