<?php

namespace SupabloxPro;

function register_block_types() {
	// TODO: Register blocks here.
}
add_action( 'init', __NAMESPACE__ . '\register_block_types' );

function add_core_block_supports( array $args, string $block_type ) {
	if ( ! str_starts_with( $block_type, 'core/' ) ) {
		return $args;
	}

	if (
		is_feature_enabled( 'responsive_core_typography' ) &&
		isset( $args['supports']['typography']['fontSize'] ) &&
		$args['supports']['typography']['fontSize']
	) {
		$args['supports']['supabloxProTypography'] = [
			'fontSize' => $args['supports']['typography']['fontSize'],
		];
		$args['supports']['typography']['fontSize'] = false;
	}

	if (
		is_feature_enabled( 'responsive_core_spacing' ) &&
		isset( $args['supports']['spacing'] ) &&
		$args['supports']['spacing']
	) {
		$args['supports']['supabloxProSpacing'] = $args['supports']['spacing'];
		$args['supports']['supabloxProSpacing']['blockGap'] = false;
		$args['supports']['spacing']['margin'] = false;
		$args['supports']['spacing']['padding'] = false;
	}

	if ( in_array( $block_type, [ 'core/cover', 'core/column', 'core/group' ], true ) ) {
		$args['supports']['supabloxProGroupHover'] = true;
	}

	$excluded_blocks = [
		'core/separator',
		'core/spacer',
		'core/query',
		'core/template-part',
	];

	if ( ! in_array( $block_type, $excluded_blocks, true ) ) {
		$args['supports']['supabloxProEffects'] = true;
		$args['supports']['supabloxProFilters'] = true;
		$args['supports']['supabloxProLayout'] = true;
		$args['supports']['supabloxProTransforms'] = true;
		$args['supports']['supabloxProAppearance'] = true;
	}

	return $args;
}
add_filter( 'register_block_type_args', __NAMESPACE__ . '\add_core_block_supports', 11, 2 );

function block_class_name( string $class_name, string $block_name ) {
	if ( str_starts_with( $block_name, 'supablox/' ) ) {
		return 'sb-block-' . substr( $block_name, 9 );
	}
	return $class_name;
}
add_filter( 'block_default_classname', __NAMESPACE__ . '\block_class_name', 10, 2 );

// TODO: Remove before 1.0
function transform_block_attributes( array $parsed_block ) {
	$style = $parsed_block['attrs']['supablox']['style'] ?? [];

	if (
		array_key_exists( 'media:sm', $style ) ||
		array_key_exists( 'media:md', $style ) ||
		array_key_exists( 'media:lg', $style )
	) {
		foreach ( $style as $media_key => $value ) {
			if ( str_starts_with( $media_key, 'media:' ) ) {
				$breakpoint = substr( $media_key, 6 );

				foreach ( $value as $sub_key => $sub_value ) {
					if ( str_starts_with( $sub_key, ':' ) ) {
						foreach ( $sub_value as $state_key => $state_value ) {
							$style[ substr( $sub_key, 1 ) ][ $state_key ][ $breakpoint ] = $state_value;
						}
					} else {
						$style['default'][ $sub_key ][ $breakpoint ] = $sub_value;
					}
				}

				unset( $style[ $media_key ] );
			}
		}

		$parsed_block['attrs']['supablox']['style'] = $style;
	}

	return $parsed_block;
}
add_filter( 'render_block_data', __NAMESPACE__ . '\transform_block_attributes' );
