<?php

namespace SupabloxPro\Packages\Admin;

class License {

	private static $instance = null;

	public function __construct() {}

	public function init(): void {}

	public function add_settings(): void {}

	public function settings_sanitize_callback( array|null $input ): array {
		return $input ?? [];
	}

	public function get_activation(): array {
		return get_option( PACKAGE_NAME . '_activation', [] );
	}

	public function update_activation( $data ): void {
		$data = wp_parse_args( $data, [ 'error' => null ] );
		update_option( PACKAGE_NAME . '_activation', $data );
	}

	public function clear_activation(): void {
		delete_option( PACKAGE_NAME . '_activation' );
	}

	public function is_activated(): bool {
		return true;
	}

	public function is_activation_valid() {
		return ! empty( $this->get_activation()['error'] );
	}

	public function set_error( string $message ): void {
		$this->update_activation( array_merge( $this->get_activation(), [ 'error' => $message ] ) );
	}

	public function validate( array $args = [] ): array|\WP_Error {
		return Supablox_Api::get_instance()->get_license( $args );
	}

	public function activate( array $args = [] ): array|\WP_Error {
		return Supablox_Api::get_instance()->activate_license( $args );
	}

	public function deactivate( array $args = [] ): array|\WP_Error {
		return Supablox_Api::get_instance()->deactivate_license( $args );
	}

	public function get_latest_version( array $args = [] ): array|\WP_Error {
		return Supablox_Api::get_instance()->get_latest_version( $args );
	}

	public function get_version( string $version, array $args = [] ): array|\WP_Error {
		return Supablox_Api::get_instance()->get_version( $version );
	}

	public static function get_instance(): static {
		if ( self::$instance === null ) {
			switch ( PACKAGE_LICENSE ) {
				case 'lemon-squeezy':
					require __DIR__ . '/class-lemon-squeezy-license.php';
					self::$instance = new Lemon_Squeezy_License();
					break;
				default:
					self::$instance = new self();
			}
			self::$instance->init();
		}
		return self::$instance;
	}
}
