<?php

namespace SupabloxPro\Packages\Admin;

class Plugin_Updater {

	public function init() {
		add_filter( 'plugins_api', [ $this, 'plugin_info' ], 20, 3 );
		add_filter( 'update_plugins_api.supablox.com', [ $this, 'get_update' ], 10, 4 );

		if ( PACKAGE_LICENSE ) {
			add_filter( 'upgrader_package_options', [ $this, 'package_options' ], 9 );
		}
	}

	public function plugin_info( $result, $action, $args ) {
		if ( $action !== 'plugin_information' ) {
			return $result;
		}

		$slug = plugin_basename( PACKAGE_FILE );

		if ( empty( $args->slug ) || $args->slug !== $slug ) {
			return $result;
		}

		$data = Supablox_Api::get_instance()->get_plugin_info();

		if ( is_wp_error( $data ) ) {
			return $result;
		}

		return (object) wp_parse_args( $data, [ 'slug' => $slug ] );
	}

	public function get_update( $update, array $plugin_data, string $plugin_file, array $locales ) {
		$slug = plugin_basename( PACKAGE_FILE );

		if ( $plugin_file !== $slug ) {
			return $update;
		}

		$data = License::get_instance()->get_latest_version();

		if ( is_wp_error( $data ) ) {
			return $update;
		}

		return wp_parse_args( $data, [ 'slug' => $slug ] );
	}

	public function package_options( array $options ) {
		if (
			empty( $options['package'] ) ||
			empty( $options['hook_extra']['plugin'] ) ||
			$options['hook_extra']['plugin'] !== plugin_basename( PACKAGE_FILE )
		) {
			return $options;
		}

		if ( preg_match( '/\/v\d+\/plugin\/' . preg_quote( PACKAGE_NAME, '/' ) . '\/(?P<version>[^\/]+)/', $options['package'], $matches ) ) {
			$data = License::get_instance()->get_version( trim( $matches['version'] ) );

			if ( is_wp_error( $data ) || empty( $data['package'] ) ) {
				$options['package'] = null;
			} else {
				$options['package'] = $data['package'];
			}
		}

		return $options;
	}
}
